#!/bin/bash
#
# CVS Info:
# $Author: markus $
# $Date: 2002/11/30 20:56:23 $
# $Revision: 1.9 $
#
# Patch description generator by Lars Ellenberg <l.g.e@web.de>
# see list_suse_updates by the same author, and
# fou4s by Markus Gaugusch, see http://fou4s.gaugusch.at/
# or probably even yast2 online update, if you can tell it to look
# at those patch descriptions in the first place.
#
# Written in bash, not perl, since Markus likes bash better :)
#
# gpd uses rsync to generate the recursive directory listings, so this
# obviously only works with rsync enabled servers...
#

# to resume a previously interupted run
RESUME=''
if [ "$1" = "--resume" ] ; then
	shift
	RESUME=yes
fi

# this test by coincidence does the right thing when $1 is not set,
# since it is not quoted
if [ ! -f $1 ] ; then
	echo "Listing file $1 not found!"
	exit 1
fi
LISTING=$1 # use an already retrieved listing and don't get it with rsync

# get fou4s common functions
# actually I do only need the variables
for fou4s in `dirname $0`/fou4s ./fou4s /usr/sbin/fou4s ; do
	test -x $fou4s && source $fou4s --commonfuncs && break
done
for mypath in `dirname $0` . /usr/bin ; do
	test -x $mypath/get_info_from_ARCHIVES.pl && break
done

if [ -z "$GPDLIST" -a -z "$LISTING" ] ; then
	echo "Please edit $CONFIGFILE and put something into GpdList!!"
	exit 1
fi

# you may want to include DESC_SERIAL and RSYNCSERVER into fou4s config
# if not defined, they are defined here.
# ARCHIVES=$ARCHIVESDIR/ARCHIVES # fixme put into dlpath/.../suseversion!/
# or ARCHIVES="ARCHIVES-$SUSEVERSION" ?
for a in $ARCHIVES ./ARCHIVES `dirname $0`/ARCHIVES /var/cache/fou4s/ARCHIVES ; do
	test -f $a && ARCHIVES=$a && break
done
if [ ! -f $ARCHIVES ] ; then
	echo
	echo "HINT: Copy ARCHIVES.gz to ${ARCHIVES%/*}, and gunzip it. This will make gpd.sh write better readable patch descriptions." | fmt
	ARCHIVES=
fi
DESC_SERIAL=G
test $VERBOSE -ge 1 && echo "Will use -$DESC_SERIAL as description serial number"
# RSYNCSERVER=rsync://ftp.gwdg.de/SuSE/ftp.suse.com/suse/$ARCH # fixme put into fou4s.conf

cat << _EOF

          Warning!
			 
          Updates from this tree are NOT necessarily 
          UPDATED,  EVEN WHEN VULNERABLE TO SECURITY 
          HOLES!

_EOF

function print_patch_description ()
{
	# purpose of the update might be in the *.info or .link/*.en files
	# which I try to get into the LONGDESC

	cat <<-__EOP__
	##
	## Patch description of patch $name
	##

	Kind: ${KIND:-generated}

	Shortdescription.english: ${SUMMARY:-no summary available}

	Longdescription.english:
	This update is available from
	$full_path

	This patch description was generated automaticaly from recursive
	directory listings by amorsu/fou4s patch description generator.

	Comments, requests for features, and bug reports, go to l.g.e(fou4s)@web.de

	${LONGDESC:-No further information available}
	Hsilgne.noitpircsedgnol:

	Size: $((size/1024))

	## file timestamp on ftp server: $date $time
	Buildtime: $unix_time

	MinYaST1Version:
	MinYaST2Version:

	Packages:
	##
	## -----> $name <-----
	##
	Filename: $name.rpm
	Label: $SUMMARY
	Series:
	InstPath: $full_path
	Size: 0815 $size
	Buildtime: $unix_time
	BuiltFrom:
	Flag:
	RpmGroup:${GROUP:+ $GROUP}
	Copyright:
	Version: $version
	StartCommand:
	Obsoletes:
	Requires:
	Provides:
	Segakcap:
	__EOP__
}

# expects path in the environment, splits it up into
# name, ver, rel, version=$ver-$rel
# -lge: path remains unchanged for now
function extract_version ()
{
	name=${path##*/}
	# path=${path%/*}
	rel=${name##*-}
	[ "$rel" == "$name" ] && return 1 # no '-' found
	rel=${rel%%.*}
	name=${name%-*}
	ver=${name##*-}
	[ "$ver" == "$name" ] && return 1 # no second '-' found
	name=${name%-*}
	version=$ver-$rel
}

function grep_most_recent_versions() {
	sed -ne '
	/^-/!d          # delete if not plain file
	/\.rpm$/!d      # or not rpm
	/\.src\.rpm$/d  # or .src.rpm
	s/^[^ ]* *//    # strip permissions
	s/ \+/'$'\t''/g # squeeze blanks into tabs
	#  anything/ name- version -    release     extension
	s/^\(.*\)\/\(.*-\)\([^-]\+-\)\([0-9]\+\)\(\.[^-]*\)$/\1|\2|'$'\\\n''\3'$'\\\n''|\4|\5/p' \
	| sed -ne '
		1~3h # hold it
		# insert space separators around version numbers
		2~3{s/\([0-9]\+\)/ \1 /g;H;}
		# and print it
		3~3{H;x;s/'$'\\\n''//g;p;}' \
	| sort -t "|" -k 4,4nr \
	| sort -t " " -s -nr -k 2,2 -k 4,4 -k 6,6 -k 8,8 -k 10,10 -k 12,12 -k 14,14 \
	| sort -t "|" -s -u -k 2,2 \
	| sed 's/|/\//; s/[ |]//g;'
}

# just for fun :)
_pgd_c=0
_pgd_total=0
function progressdot() {
	if [ $VERBOSE -ge 1 ] ; then
		echo $*
	elif [ -t 1 ] ; then
		if [ $# -eq 0 ] ; then
			printf "%$(( 60 - _pgd_c - (_pgd_c+9)/ 10 ))d\n" $_pgd_total
			_pgd_total=0
			_pgd_c=0
		else
			if [ $_pgd_c -ge 50 ] ; then
				printf "%5d\n" $_pgd_total
				_pgd_c=0
			fi
			test ${_pgd_c: -1:1} = 0 && echo -n " "
			echo -n .
			let _pgd_c="_pgd_c+1"
			let _pgd_total="_pgd_total+1"
		fi
	fi
}

# do not use [[ ]] for compatibility
function parse_listing ()
{
	local size date time full_path path name ver rel
	local version unix_time
	local hash_part rpm_info LONGDESC SUMMARY GOUP 

	server=${RSYNCSERVER#rsync://}
	server=${server%%/*}
	if [ -z "$RESUME" ] ; then
		echo "Removing previously generated patch descriptions"
		rm $DLPATH/$server/$SERVERPATH/patches/*-$DESC_SERIAL 2>/dev/null
	fi

	echo "Generating new patch descriptions to $DLPATH/$server/$SERVERPATH"

	# only consider the most recent version per basename
	grep_most_recent_versions |
	{ # this brace is only for the total status of progressdot :)
	while read size date time path ; do
		if [ ${path#update/$SUSEVERSION/} != $path ] ; then
			# this is for the update trees
			# should come up with something similar as the
			# descriptions in the patches{,.cont} directories
			full_path=${path#update/$SUSEVERSION/}
		else
			# have to use path relative to default update directory
#FIXME what about 8.1? 
			full_path=../../$path
		fi
		# if we cannot guess the version, ignore this file
		extract_version || continue
		test $VERBOSE -ge 1 && printf "%-40s  %-16s %s\n" $name $ver-$rel ${path%/*}

		if [ -e "$DLPATH/$server/$SERVERPATH/patches/$name-$DESC_SERIAL" ] ; then
			progressdot Skip existing $name-$DESC_SERIAL
			continue
		fi

		# is --utc a correct assumption?
		# it is a rough guess, anyways...
		unix_time=`date --utc -d "$date $time" +%s`
		LONGDESC=''
		SUMMARY=''
		GROUP=''
		KIND=''
		rpm_info=''

		# try to get some info about this update
		# ??? are there other variants of info files ???
		#
		for info in "${path%.rpm}_en.info" "${path%/*}/.link/$name.en"; do
			fgrep -q "$info" $LISTING || continue
			LONGDESC=$(wget $GLOBALWGETOPTS -q -O - $SERVER/$ARCH/$info)
			break
		done

		if [ -e "$ARCHIVES" ] ; then
			# echo calling get_info with $name
			INFO=`$mypath/get_info_from_ARCHIVES.pl $name 2>/dev/null`
			SUMMARY="${INFO%%|*}"
			: ${LONGDESC:="${INFO#*|}"}
		else
			rpm_info=`rpm -q --queryformat "%{SUMMARY}|%{GROUP}" $name 2>/dev/null`
			if [ "$rpm_info" ] ; then
				SUMMARY=${rpm_info%%|*}
				GROUP=${rpm_info#*|}
			fi
		fi
		case "$LONGDESC" in
			# TODO add all kind of patterns here
			*[Ss]ecurity\ update*) KIND=security;;
		esac
		# echo $name $version $full_path $size $unix_time $SUMMARY $GROUP
		progressdot Generating $name-$DESC_SERIAL
		print_patch_description > $DLPATH/$server/$SERVERPATH/patches/$name-$DESC_SERIAL
	done
	progressdot
	}
}

# if ! mkdir -p $DLPATH/$server/$SERVERPATH/patches ; then
if [ ! -d $DLPATH/$server/$SERVERPATH/patches ] ; then
	echo "No directory $DLPATH/$server/$SERVERPATH/patches"
	echo "Please run/configure fou4s to create it auomatically. (fou4s -u)"
#fixme let fou4s do checking??
	exit 1
fi

start_time=`date +%s`
if [ -n "$LISTING" ] ; then
	parse_listing < $LISTING
else
	LISTING=`mktemp $DLPATH/tmp.rsync.list.XXXXXX`
	if [ ! -f "$LISTING"  ] ; then
		echo "Error creating tempfile in $DLPATH!!"
		exit 2
	fi
	# how about: trap "rm $LISTING" EXIT ... ... ?

	echo "# $ARCH $SUSEVERSION $RSYNCSERVER $DLPATH/$server/$SERVERPATH/patches" > $LISTING
	echo Connecting to rsync server $RSYNCSERVER/
	for PACKAGE_PATH in $GPDLIST ; do
		echo "# $PACKAGE_PATH" >> $LISTING
		echo "Getting descriptions from $PACKAGE_PATH/"
		rsync -r $RSYNCSERVER/$PACKAGE_PATH/ \
		| grep -v kernel | grep -v " zq1/" \
		| sed 's/^\(.*\) \(.*\)$/\1 '${PACKAGE_PATH//\//\\/}'\/\2/' >> $LISTING
		ret=${PIPESTATUS[0]}
		# seems like rsync returns 11 for server response "no such directory"
		# some of the example locations are not available for all
		# arch/releases, so just warn
		test $ret -eq 0 -o $ret -eq 11 || exit -1
		test $ret -eq 11 && echo "$PACKAGE_PATH NOT FOUND!"
	done
	end_time=`date +%s`
	echo Downloaded in $((end_time-start_time)) seconds
	start_time=$end_time
	parse_listing < $LISTING
	test $VERBOSE -ge 2 && echo "Kept $LISTING" \
	|| rm $LISTING
fi
end_time=`date +%s`
echo Generated in $((end_time-start_time)) seconds
